-- Import glue module --------------------------------------------------------------------------------------------------
package.path = package.path .. ";" .. string.match(({reaper.get_action_context()})[2], "(.-)([^\\/]-%.?([^%.\\/]*))$") .. "?.lua"
local glue = require "GlueTools"

-----------------
-- Mouse table --
-----------------

local mouse = {  
                  -- Constants
                  LB = 1,
                  RB = 2,
                  CTRL = 4,
                  SHIFT = 8,
                  ALT = 16,
                  
                  -- "cap" function
                  cap = function (mask)
                          if mask == nil then
                            return gfx.mouse_cap end
                          return gfx.mouse_cap&mask == mask
                        end,
                          
                  uptime = 0,
                  
                  last_x = -1, last_y = -1,
                 
                  dx = 0,
                  dy = 0,
                  
                  ox_l = 0, oy_l = 0,    -- left click position
                  ox_r = 0, oy_r = 0,    -- right click position
                  capcnt = 0,
                  last_LMB_state = false
               }
              
--////////////////////////////////////////////////////////////////


---------------
-- class.lua --
---------------
-- base class for all elements

------------- "class.lua" is copied from http://lua-users.org/wiki/SimpleLuaClasses -----------

-- class.lua
-- Compatible with Lua 5.1 (not 5.0).
function class(base, init)
   local c = {}    -- a new class instance
   if not init and type(base) == 'function' then
      init = base
      base = nil
   elseif type(base) == 'table' then
    -- our new class is a shallow copy of the base class!
      for i,v in pairs(base) do
         c[i] = v
      end
      c._base = base
   end
   -- the class will be the metatable for all its objects,
   -- and they will look up their methods in it.
   c.__index = c

   -- expose a constructor which can be called by <classname>(<args>)
   local mt = {}
   mt.__call = function(class_tbl, ...)
   local obj = {}
   setmetatable(obj,c)
   if init then
      init(obj,...)
   else 
      -- make sure that any stuff from the base class is initialized!
      if base and base.init then
      base.init(obj, ...)
      end
   end
   return obj
   end
   c.init = init
   c.is_a = function(self, klass)
      local m = getmetatable(self)
      while m do 
         if m == klass then return true end
         m = m._base
      end
      return false
   end
   setmetatable(c, mt)
   return c
end
----------------------------------------------------------------------------------------


-----------------------
-- Check box class --
-----------------------

local Created_checkboxes = {} -- All created checkboxes are automatically collected to this table.

-----------------------
-- "class variables" --
-----------------------

local Checkbox_settings = 
  {
    horizontal_spacing = 5,       -- default horizontal spacing between checkboxes
    vertical_spacing = 5,         -- default vertical spacing between checkboxes
    name_horizontal_spacing = 5,  -- default spacing between checkbox and its name
    rightmost_pos = -1,           -- current rightmost x coordinate is stored to this variable
    topmost_pos = -1,             -- current topmost y coordinate is stored to this variable
    
    -- for auto-positioning
    default_x = 10,               -- if x1 is "nil", drawing starts at this x position
    default_y = 10                -- if y1 is "nil", drawing starts at this y position
  }


local Checkbox = class(
                 function(cb, x1, y1, size, state, name)
                   if #Created_checkboxes == 0 then
                     cb.x1     = x1 or Checkbox_settings.default_x
                     cb.y1     = y1 or Checkbox_settings.default_y
                     cb.size   = size or gfx.texth
                     cb.state  = state or false
                     cb.name   = name or "1"
                     cb.id     = 1
                     Created_checkboxes[1] = cb
                     
                   else -- at least one checkbox is created ("Created_checkboxes" is not empty)
                     cb.x1 = x1 or Created_checkboxes[#Created_checkboxes].x1
                     cb.y1 = y1 or Created_checkboxes[#Created_checkboxes].y1 + 
                                   Created_checkboxes[#Created_checkboxes].size + 
                                   Checkbox_settings.vertical_spacing
                             
                     cb.size  = size or gfx.texth
                     cb.state = state or false
                     cb.name = name or #Created_checkboxes + 1
                     cb.id = id or #Created_checkboxes + 1
                     Created_checkboxes[#Created_checkboxes + 1] = cb -- append current instance to "Created_checkboxes" -table
                   end
                   
                  
                   cb.name_w, cb.name_h = gfx.measurestr(cb.name)
                   cb.name_x = cb.x1 + cb.size + Checkbox_settings.name_horizontal_spacing
                   cb.name_y = cb.y1 + 0.5*(cb.size) - 0.5*cb.name_h

                   cb._mouse_on = false
                   cb._clicked = false
                     
                 end
)


-------------
-- Methods --
-------------

function Checkbox:get_checkbox_table()
  return Created_checkboxes
end


function Checkbox:get_settings_table()
  return Checkbox_settings
end


function Checkbox:get_name_state_table()
  local t = {}
  local cbs = Created_checkboxes
  for i=1, #cbs do
    t[i] = {}
    t[i].name = cbs[i].name
    t[i].state = cbs[i].state
  end
  return t
end


-- Set vertical spacing
function Checkbox:set_vert_spacing(new_spacing)
  Checkbox_settings.vertical_spacing = new_spacing
end


-- Set horizontal spacing for checkbox name (between checkbox and its name)
function Checkbox:set_horiz_spacing_for_name(new_spacing)
  Checkbox_settings.name_horizontal_spacing = new_spacing
end


-- Check "mouse on checkbox" state
function Checkbox:mouse_on()
  if gfx.mouse_x > self.x1 and gfx.mouse_x < self.x1 + self.size and gfx.mouse_y > self.y1 and gfx.mouse_y < self.y1 + self.size then
    self._mouse_on = true
    return true
  else
    self._mouse_on = false
    return false
  end
end


function Checkbox:create_from_name_state_table(name_state_table)
  local t = {}
  local nst = name_state_table
  for i=1, #nst do
    t[i] = Checkbox(start_x, start_y, nil, nst[i].state, nst[i].name)
  end
  return t
end
  

-- Update and draw all created checkboxes
function Checkbox:update_all()
  local cbs = Created_checkboxes
  
  for i=1, #cbs do
    if cbs[i]:mouse_on() and mouse.last_LMB_state == false and mouse.cap(mouse.LB) then
      cbs[i]._clicked = true
    end
    local x1,y1,size = cbs[i].x1, cbs[i].y1, cbs[i].size
    local state, name = cbs[i].state, cbs[i].name
    gfx.set(0.8,0.8,0.8,1)
    gfx.rect(x1, y1, size, size, 0)
    
    -- Checkbox is checked
    if cbs[i].state then
      gfx.set(0.8,0.8,0.8,1)
      -- Inner rectangle
      gfx.rect(x1+3, y1+3, size-6, size-6, 1) -- smaller rectangle
    end
    
    -- Draw name
    gfx.x = cbs[i].name_x
    gfx.y = cbs[i].name_y
    gfx.drawstr(name)
    
    ----if not mouse.cap(mouse.LB) then -- (change state when LMB is released)
      if cbs[i]._clicked then -- and cbs[i]._mouse_on then
        if cbs[i].on_click ~= nil then cbs[i]:on_click()
          --cbs[i]:on_click()
        end
        cbs[i].state = not cbs[i].state
      end
      cbs[i]._clicked = false
    ----end
  end
end


---------------------------------------------------------------------------------


------------------
-- Button class --
------------------

--NOTE: This is old, but...well..working

local Button = class(
                      function(btn,x1,y1,w,h,state_count,state,visual_state,lbl,help_text)
                        btn.x1 = x1
                        btn.y1 = y1
                        btn.w = w
                        btn.h = h
                        btn.x2 = x1+w
                        btn.y2 = y1+h
                        btn.state = state
                        btn.state_count = state_count - 1
                        btn.vis_state = visual_state
                        btn.label = lbl
                        btn.help_text = help_text
                        btn.__mouse_state = 0
                        btn.label_w, btn.label_h = gfx.measurestr(btn.label)
                        btn.__state_changing = false
                        btn.r = 0.7
                        btn.g = 0.7
                        btn.b = 0.7
                        btn.a = 0.2
                        btn.lbl_r = 1
                        btn.lbl_g = 1
                        btn.lbl_b = 1
                        btn.lbl_a = 1
                      end
                    )


-------------
-- Methods --
-------------

-- returns true if "mouse on element"
function Button:__is_mouse_on()
  return(gfx.mouse_x > self.x1 and gfx.mouse_x < self.x2 and gfx.mouse_y > self.y1 and gfx.mouse_y < self.y2)
end


function Button:__lmb_down()
  return(mouse.last_LMB_state == false and gfx.mouse_cap & 1 == 1 and self.__mouse_state == 0)
  --return(last_mouse_state == 0 and self.mouse_state == 1)
end


function Button:set_help_text()
  if self.help_text == "" then return false end
    gfx.set(1,1,1,1)
    gfx.x = 10
    gfx.y = 10
    gfx.printf(self.help_text)
end


function Button:set_color(r,g,b,a)
  self.r = r
  self.g = g
  self.b = b
  self.a = a
end


function Button:set_label_color(r,g,b,a)
  self.lbl_r = r
  self.lbl_g = g
  self.lbl_b = b
  self.lbl_a = a
end


function Button:draw_label()
  -- Draw button label
  if self.label ~= "" then
    gfx.x = self.x1 + math.floor(0.5*self.w - 0.5 * self.label_w) -- center the label
    gfx.y = self.y1 + 0.5*self.h - 0.5*gfx.texth

    if self.__mouse_state == 1 then 
      gfx.y = gfx.y + 1
      gfx.a = self.lbl_a*0.5
    elseif self.__mouse_state == 0 then
      gfx.a = self.lbl_a
    end
  
    gfx.set(self.lbl_r,self.lbl_g,self.lbl_b,self.lbl_a)
    
    gfx.printf(self.label)
    if self.__mouse_state == 1 then gfx.y = gfx.y - 1 end
  end
end


-- Draw element (+ mouse handling)
function Button:draw()
  
  -- lmb released (and was clicked on element)
  if mouse.cap(mouse.LB) == false and self.__mouse_state == 1 then self.__mouse_state = 0 end
  
  
  -- Mouse is on element -----------------------
  if self:__is_mouse_on() then 
    if self:__lmb_down() then -- Left mouse btn is pressed on button
    --if last_mouse_state == 0 and gfx.mouse_cap & 1 == 1 and self.mouse_state == 0 then
      self.__mouse_state = 1
      if self.__state_changing == false then
        self.__state_changing = true
      else self.__state_changing = true
      end
    end
    
    self:set_help_text() -- Draw info/help text (if 'help_text' is not "")
    
    if mouse.last_LMB_state == false and gfx.mouse_cap & 1 == 0 and self.__state_changing == true then
      if self.onClick ~= nil then self:onClick()
        self.__state_changing = false
      else self.__state_changing = false
      end
    end
  
  -- Mouse is not on element -----------------------
  else
    if mouse.last_LMB_state == false and self.__state_changing == true then
      self.__state_changing = false
    end
  end  
  --gfx.a = self.a
  
  if self.__mouse_state == 1 or self.vis_state == 1 or self.__state_changing then
    --self.a = math.max(self.a - 0.2, 0.2)
    --gfx.set(0.8,0,0.8,self.a)
    gfx.set(0.8*self.r,0.8*self.g,0.8*self.b,math.max(self.a - 0.2, 0.2)*0.8)
    gfx.rect(self.x1, self.y1, self.w, self.h)

  -- Button is not pressed
  elseif not self.state_changing or self.vis_state == 0 or self.__mouse_state == 0 then
    gfx.set(self.r+0.2,self.g+0.2,self.b+0.2,self.a)
    gfx.rect(self.x1, self.y1, self.w, self.h)
   
    gfx.a = math.max(0.4*self.a, 0.6)
    -- light - left
    gfx.line(self.x1, self.y1, self.x1, self.y2-1)
    gfx.line(self.x1+1, self.y1+1, self.x1+1, self.y2-2)
    -- light - top
    gfx.line(self.x1+1, self.y1, self.x2-1, self.y1)
    gfx.line(self.x1+2, self.y1+1, self.x2-2, self.y1+1)

    --gfx.set(0.4,0,0.4,1)
    gfx.set(0.3*self.r,0.3*self.g,0.3*self.b,math.max(0.9*self.a,0.8))
    -- shadow - bottom
    gfx.line(self.x1+1, self.y2-1, self.x2-2, self.y2-1)
    gfx.line(self.x1+2, self.y2-2, self.x2-3, self.y2-2)
    -- shadow - right
    gfx.line(self.x2-1, self.y2-1, self.x2-1, self.y1+1)
    gfx.line(self.x2-2, self.y2-2, self.x2-2, self.y1+2)
  end
 
  self:draw_label()
end



---------------------------
-- Mouse event handling  --
-- (Schwa's GUI example) --
---------------------------

function OnMouseDown(x, y)
  mouse.capcnt = 0
  mouse.ox_l, mouse.oy_l = x, y
end


function OnMouseUp(x, y)
  mouse.uptime = os.clock()
  mouse.dx = 0
  mouse.dy = 0
  mouse.last_LMB_state = false
end


function OnMouseDoubleClick(x, y)
  -- handle mouse double click here
end


function OnMouseMove(x, y)
  -- handle mouse move here, use mouse.down and mouse.capcnt
  mouse.last_x, mouse.last_y = x, y
  mouse.dx = gfx.mouse_x - mouse.ox_l
  mouse.dy = gfx.mouse_y - mouse.oy_l
  mouse.capcnt = mouse.capcnt + 1
end


--------------
-- Mainloop --
--------------

function mainloop()
    
  -- (Schwa's GUI example)
  local mx, my = gfx.mouse_x, gfx.mouse_y
  if mouse.cap(mouse.LB) then              -- LMB pressed down?
    if mouse.last_LMB_state == false then  -- prevent "polling"...
      OnMouseDown(mx, my)                  --   ...run this once per LMB click
      if mouse.uptime and os.clock() - mouse.uptime < 0.20 then
        OnMouseDoubleClick(mx, my)
      end
    elseif mx ~= mouse.last_x or my ~= mouse.last_y then
      OnMouseMove(mx, my)
    end
  elseif mouse.last_LMB_state then
    OnMouseUp(mx, my)
  end

  --------------
  -- Draw GUI --
  --------------
  
  -- Draw all created checkboxes from "Created_checkboxes" -table
  -- (see "checkbox class.lua")
  Checkbox:update_all()
  
  -- draw/update button
  glue_btn:draw()
  
  if mouse.cap(mouse.LB) and mouse.last_LMB_state == false then
    mouse.last_LMB_state = true
  end
  
  gfx.update()
  if gfx.getchar() >= 0 then reaper.defer(mainloop) end
end


----------
-- Init --
----------

local gui = {}

function init()
  -- GUI table --------------------------------------------
  --   contains GUI related settings (some basic user definable settings), initial values etc.
  ---------------------------------------------------------
  
  -- Add "settings" table to "gui" table              
  gui.settings = {}
  gui.settings.font_size = 15      -- font size
  gui.settings.docker_id = 256      -- try 0, 1, 257, 513, 1027 etc.
  
  
  --------------------
  -- Initialize GUI --
  --------------------
  
  gfx.init("Breeder - Glue tool", 220, 250, gui.settings.docker_id)
  gfx.setfont(1,"Arial", gui.settings.font_size)
  gfx.clear = 3355443  -- matches with "FUSION: Pro&Clean Theme :: BETA 01" http://forum.cockos.com/showthread.php?t=155329
  -- (Double click in ReaScript IDE to open the link)
    
    
  -----------------------
  -- Create checkboxes --
  -----------------------
  
  -- Checkboxes are later created from this table 
  local glue_items_settings = {
                                {name = "Process items separately",   state = false},
                                {name = "Ignore time selection",      state = false},
                                {name = "Include fades",              state = false},
                                {name = "Preserve muted MIDI events", state = false},
                                {name = "Preserve color",             state = false},
                                {name = "Preserve names",             state = false},
                                {name = "Preserve snap offset",       state = false},
                                {name = "Preserve item notes",        state = false}
                              }
                        
                                           -- (see "Checkbox_settings" in "checkbox class")
  Checkbox:set_vert_spacing(5)             -- Optional vertical spacing between checkboxes
  Checkbox:set_horiz_spacing_for_name(5)   -- Optional horizontal spacing between checkbox and its name
  
  
  ---------------------------------------------------------
  -- Create checkboxes from "glue_items_settings" -table --
  ---------------------------------------------------------
  
  Checkbox:create_from_name_state_table(glue_items_settings)
   
   
  --------------------------
  -- Create "Glue" button --
  --------------------------
  
  -- update y position to last checkbox 
  local y = Created_checkboxes[#Created_checkboxes].y1
            + Created_checkboxes[#Created_checkboxes].size
            + gui.settings.font_size*2
            
  glue_btn = Button(10, y, 60, gui.settings.font_size + 4, 2, 0, 0, "Glue", "")
  
  -- execute "glue.GlueItems" when the Glue -button is pressed
  glue_btn.onClick = 
    function()
      local cb = Checkbox:get_name_state_table() -- (see "checkbox class")
      glue.GlueItems(true, 
                     cb[1].state, -- Process items separately
                     cb[2].state, -- Ignore time selection
                     cb[3].state, -- Include fades
                     cb[4].state, -- Preserve muted MIDI events
                     cb[5].state, -- Preserve color
                     cb[6].state, -- Preserve names
                     cb[7].state, -- Preserve snap offset
                     cb[8].state  -- Preserve item notes
                    )
    end

  mainloop()
end

init()
